@@ -31,7 +31,7 @@ end
 
 
 function _M.access(conf, ctx)
-    return limit_count.rate_limit(conf, ctx)
+    return limit_count.rate_limit(conf, ctx, plugin_name, 1)
 end
 
 
